package com.example.sam4s;

import com.example.sam4s.R;
import com.sam4s.printer.Sam4sBuilder;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager.LayoutParams;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.TextView;

public class cl_Barcode extends Activity {
	static final int SEND_TIMEOUT = 10 * 1000;
	static final int HRI_NONE_INDEX = 0;
	static final int HRI_ABOVE_INDEX = 1;
	static final int HRI_BELOW_INDEX = 2;
	static final int HRI_BOTH_INDEX = 3;
	static final int FONT_A_INDEX = 0;
	static final int FONT_B_INDEX = 1;

	boolean changeBarcodeData = false;
	String defaultBarcodeData = "";
	boolean exitActivity = false;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.barcode);

		// check close port
		if (cl_Menu.mPrinter == null) {
			finish();
			return;
		}

		// init type list
		Spinner spinner = (Spinner) findViewById(R.id.spinner_type);
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item);
		adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		int[] typelist = { R.string.bartype_upca, R.string.bartype_upce,
				R.string.bartype_ean13, R.string.bartype_jan13,
				R.string.bartype_ean8, R.string.bartype_jan8,
				R.string.bartype_code39, R.string.bartype_itf,
				R.string.bartype_codabar, R.string.bartype_code93,
				R.string.bartype_code128, };
		for (int i = 0; i < typelist.length; i++) {
			adapter.add(getString(typelist[i]));
		}
		spinner.setAdapter(adapter);

		// init hri list
		spinner = (Spinner) findViewById(R.id.spinner_hri);
		adapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item);
		adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		adapter.add(getString(R.string.hri_none));
		adapter.add(getString(R.string.hri_above));
		adapter.add(getString(R.string.hri_below));
		adapter.add(getString(R.string.hri_both));
		spinner.setAdapter(adapter);

		// init font list
		spinner = (Spinner) findViewById(R.id.spinner_font);
		adapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item);
		adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		adapter.add(getString(R.string.font_a));
		adapter.add(getString(R.string.font_b));
		spinner.setAdapter(adapter);

		// default setting
		TextView text = (TextView) findViewById(R.id.editText_modulesize_w);
		text.setText("3");

		text = (TextView) findViewById(R.id.editText_modulesize_h);
		text.setText("162");

		spinner = (Spinner) findViewById(R.id.spinner_hri);
		spinner.setSelection(2);

		setDefaultBarcodeData();

		// Registration ClickListener
		Button btnPrint = (Button) findViewById(R.id.button_print);
		btnPrint.setOnClickListener(new btnPrintClick());

		// register OnCheckedChangeListener
		spinner = (Spinner) findViewById(R.id.spinner_type);
		spinner.setOnItemSelectedListener(new SpinerOnItemSelected());

		// hide keyboard
		this.getWindow().setSoftInputMode(
				LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		if (!exitActivity) {
			try {
				cl_Menu.mPrinter.closePrinter();
			} catch (Exception e) {				
				e.printStackTrace();
			}
		}
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			exitActivity = true;
		}

		return super.onKeyDown(keyCode, event);
	}

	class btnPrintClick implements View.OnClickListener {
		@Override
		public void onClick(View v) {
			printBarcode();
		}
	}

	class SpinerOnItemSelected implements OnItemSelectedListener {
		@Override
		public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2,
				long arg3) {
			// check default change
			TextView text = (TextView) findViewById(R.id.editText_data);
			if (defaultBarcodeData.compareTo(text.getText().toString()) != 0) {
				changeBarcodeData = true;
			}
			if (changeBarcodeData) {
				// not update
				return;
			}

			// update barcode data
			setDefaultBarcodeData();
		}

		@Override
		public void onNothingSelected(AdapterView<?> arg0) {
			return;
		}

	}

	private void printBarcode() {
		TextView text = (TextView) findViewById(R.id.editText_data);
		if (text.getText().toString().isEmpty()) {
			cl_Main.showMsg("No data Error", this);
			return;
		}

		Sam4sBuilder builder = null;

		try {
			// create builder
			Intent intent = getIntent();
			// Builder
			builder = new Sam4sBuilder(intent.getStringExtra("printername"),
					intent.getIntExtra("language", 0));

			// add command
			// addBarcode
			builder.addBarcode(getBuilderData(), getBuilderType(),
					getBuilderHri(), getBuilderFont(), getBuilderWidth(),
					getBuilderHeight());

			// send builder data
			try {
				cl_Menu.mPrinter.sendData(builder);
			} catch (Exception e) {
				e.getStackTrace();
			}
		} catch (Exception e) {
			cl_Main.showMsg(e.getLocalizedMessage(), this);
		}

		// remove builder
		if (builder != null) {
			try {
				builder.clearCommandBuffer();
				builder = null;
			} catch (Exception e) {
				builder = null;
			}
		}
	}

	private String getBuilderData() {
		TextView text = (TextView) findViewById(R.id.editText_data);
		return text.getText().toString();
	}

	private int getBuilderType() {
		Spinner spinner = (Spinner) findViewById(R.id.spinner_type);
		int[] typelist = { Sam4sBuilder.BARCODE_UPC_A,
				Sam4sBuilder.BARCODE_UPC_E, Sam4sBuilder.BARCODE_EAN13,
				Sam4sBuilder.BARCODE_JAN13, Sam4sBuilder.BARCODE_EAN8,
				Sam4sBuilder.BARCODE_JAN8, Sam4sBuilder.BARCODE_CODE39,
				Sam4sBuilder.BARCODE_ITF, Sam4sBuilder.BARCODE_CODABAR,
				Sam4sBuilder.BARCODE_CODE93, Sam4sBuilder.BARCODE_CODE128, };
		if (spinner.getSelectedItemPosition() < typelist.length) {
			return typelist[spinner.getSelectedItemPosition()];
		} else {
			return typelist[0];
		}
	}

	private int getBuilderHri() {
		Spinner spinner = (Spinner) findViewById(R.id.spinner_hri);
		switch (spinner.getSelectedItemPosition()) {
		case HRI_ABOVE_INDEX:
			return Sam4sBuilder.HRI_ABOVE;
		case HRI_BELOW_INDEX:
			return Sam4sBuilder.HRI_BELOW;
		case HRI_BOTH_INDEX:
			return Sam4sBuilder.HRI_BOTH;
		case HRI_NONE_INDEX:
		default:
			return Sam4sBuilder.HRI_NONE;
		}
	}

	private int getBuilderFont() {
		Spinner spinner = (Spinner) findViewById(R.id.spinner_font);
		switch (spinner.getSelectedItemPosition()) {
		case FONT_B_INDEX:
			return Sam4sBuilder.FONT_B;
		case FONT_A_INDEX:
		default:
			return Sam4sBuilder.FONT_A;
		}
	}

	private int getBuilderWidth() {
		TextView text = (TextView) findViewById(R.id.editText_modulesize_w);
		try {
			return Integer.parseInt(text.getText().toString());
		} catch (Exception e) {
			return 0;
		}
	}

	private int getBuilderHeight() {
		TextView text = (TextView) findViewById(R.id.editText_modulesize_h);
		try {
			return Integer.parseInt(text.getText().toString());
		} catch (Exception e) {
			return 0;
		}
	}

	private void setDefaultBarcodeData() {
		// update default data
		defaultBarcodeData = getDefaultBarcodeData();

		// update ui
		TextView text = (TextView) findViewById(R.id.editText_data);
		text.setText(defaultBarcodeData);
	}

	private String getDefaultBarcodeData() {
		Spinner spinner = (Spinner) findViewById(R.id.spinner_type);
		int defaultDataNo = 0;
		int[] dataList = { R.string.bardata_upca, R.string.bardata_upce,
				R.string.bardata_ean13, R.string.bardata_jan13,
				R.string.bardata_ean8, R.string.bardata_jan8,
				R.string.bardata_code39, R.string.bardata_itf,
				R.string.bardata_codabar, R.string.bardata_code93,
				R.string.bardata_code128, R.string.bardata_gs1128, };
		if (spinner.getSelectedItemPosition() < dataList.length) {
			defaultDataNo = dataList[spinner.getSelectedItemPosition()];
		} else {
			defaultDataNo = dataList[0];
		}

		return getString(defaultDataNo);
	}
}
